def construct_shell_thickness(mesh,groupstr,info):
  '''construct a CA command for variable thickness of a shell
  
  The function determines a shell thickness based on the coordinates of the elements
  and outputs a dictionary ThicknessGroupShell with element numbers and corresponding thickness.
  This dictionary may be used in the CAster command 
  shellch=AFFE_CARA_ELEM(MODELE=cocModel,COQUE=(ThicknessGroupShell),)
  as a replaced for
  shellch1=AFFE_CARA_ELEM(MODELE=cocModel,COQUE=_F(GROUP_MA='shell',EPAIS=thickness,),);
  '''
  from Utilitai.partition import *
  import numpy

  PyMesh   = MAIL_PY()
  PyMesh.FromAster(mesh);
  nonu     = PyMesh.dime_maillage[0]
  elnu     = PyMesh.dime_maillage[2]
  ncoord   = PyMesh.cn
  connect  = PyMesh.co
  NodeList = list(PyMesh.correspondance_noeuds)
  ElemList = list(PyMesh.correspondance_mailles)
  ElemType = list(PyMesh.tm)

  NodeGroup = PyMesh.gno               ## list of GROUP_NO groups, eg groupstr
  ElemGroup = PyMesh.gma               ## list of GROUP_MA groups, eg groupstr
  ShellGroup = ElemGroup[groupstr]     ## selected group ''groupstr''
  print 'ShellGroup: ',ShellGroup

  ThicknessGroupShell = ShellThicknessAverageYnode(ElemList,ShellGroup,connect,ncoord,info)
  #ThicknessGroupShell = ShellThicknessCentreYnode(ElemList,ShellGroup,connect,ncoord,info)
  if info>=1:
    print 'thicknessGroupShell: ',ThicknessGroupShell

  return ThicknessGroupShell
  
  
  
def ShellThicknessAverageYnode(ElementList,ShellGroup,ElementConnect,NodeCoord,info):
   '''determine thickness of a shell based on average y coordinates of an element'''


   print 'ShellGroup: ',ShellGroup

   ThicknessGroupShell=[]                   ## initialise PressureOnShell dictionary
   for ii in xrange(len(ShellGroup)):
      ElementNumber = ShellGroup[ii]
      ncount = len(ElementConnect[ii])        #### determine average y coordinate of elemenent
      sumxyz = [0.0,0.0,0.0]
      for jj in xrange(0,ncount):       # last node is centre nodes - may be ommited -->ncount-1
        Gnode = ElementConnect[ii][jj]    # globalnode = connect[elemii,internal_nodejj]
        sumxyz += NodeCoord[Gnode]    # this might be written more compact --> for later
      sumxyz /=ncount
      
      # define your own code for thickness here 
      # thickness now depends on average y coordinate
      tempth = sumxyz[1]*0.01+0.1  


      ThicknessGroupShell.append({'EPAIS':tempth,'MAILLE':'M%d'%(ElementNumber+1)});  ## add 1 for element number Mxx
      ##print shellcount, tempth 
   return ThicknessGroupShell






def ShellThicknessCentreYnode(ElementList,ShellGroup,ElementConnect,NodeCoord,info):
  '''determine thickness of a shell based on centre y coordinate of an element'''

  for ii in xrange(len(ShellGRoup)):
      ElementNumber = ShellGroup[ii]

      ### use centre node of element (tested for quad9 only, should work for tria7 aswell:
      CentreNode = ElemConnect[ElementNumber][-1]       ### centre node coordinate stored in last ElementConnnect ie [-1]
      Centrexyz = NodeCoord[CentreNode]
      yCentreCoord = Centrexyz[1]    #  xcoord[0] ycoord[1] zcoord[2]


      # define your own code for thickness here 
      # thickness now depends on centre y coordinate only
      tempth = yCentreCoord*0.1+0.1


      ThicknessGroupShell.append({'EPAIS':tempth,'MAILLE':'M%d'%(ElementNumber+1)});  ## add 1 for element number MXX for CAmesh
      #FORCE_COQUE=_F(GROUP_MA='shell',PRES=pressure,),);

      if info>=1:
        print 'element: %d'%ElementNumber+' centre node number: %d'%CentreNode+' y coordinate: %f'%yCentreCoord+' pressure on element: %f'%temppress

  return ThicknessGroupShell








def ApplyPressureOnGroup(mesh,groupstr,info):
   '''Calculate a variable pressure on shell elements

   The function determines a pressure on each element in the group groupstr depending on the 
   centre y coordinate of the element.
   Adjust this for your own needs.
   '''
   from Utilitai.partition import *

   PyMesh   = MAIL_PY()         # convert CodeAster mesh to Python
   PyMesh.FromAster(mesh);
   nonu     = PyMesh.dime_maillage[0]   ## number of nodes
   elnu     = PyMesh.dime_maillage[2]   ## number of elements
   NodeCoord    = PyMesh.cn             ## xyz coordinates of nodes (nonu*3 matrix)
   ElemConnect  = PyMesh.co             ## node numbers of elements (elnu*x matrix)
   NodeList = list(PyMesh.correspondance_noeuds)
   ElemList = list(PyMesh.correspondance_mailles)
   ElemType = list(PyMesh.tm)
   NodeGroup = PyMesh.gno               ## list of GROUP_NO groups
   ElemGroup = PyMesh.gma               ## list of GROUP_MA groups, eg groupstr

   if info>=1:
     print 'groups by argument:%s'%(groupstr)
     print ElemGroup[groupstr]
     
 
   PressureOnShell=[]                   ## initialise PressureOnShell dictionary
   for ii in xrange(len(ElemGroup[groupstr])):
      ElementNumber = ElemGroup[groupstr][ii]
      #print 'ElemGroup: ',ii,ElementNumber
      #tempstr = 'connection information of element %d:'%ii+' element number %d '%ElementNumber
      #print tempstr,ElemConnect[ElementNumber]

      ### use centre node of element (tested for quad9 only, should work for tria7 aswell:
      CentreNode = ElemConnect[ElementNumber][-1]       ### centre node coordinate stored in last ElementConnnect
      Centrexyz = NodeCoord[CentreNode]
      yCentreCoord = Centrexyz[1]    # [0] xcoord [1] ycoord [2] zcoord


      # make thickness depending on y coordinate (1) of centre node only
      # define your own code here
      #temppress = (2.5-yCentreCoord)*1000  
      temppress = yCentreCoord*1000  


      PressureOnShell.append({'PRES':temppress,'MAILLE':'M%d'%(ElementNumber+1)});  ## add 1 for element number Mxx (from PyMesh to CAmesh)
      #FORCE_COQUE=_F(GROUP_MA='shell',PRES=pressure,),);
      if info>=1:
        print 'element: %d'%ElementNumber+' centre node number: %d'%CentreNode+' y coordinate: %f'%yCentreCoord+' pressure on element: %f'%temppress
   if info>=1:
    print 'ApplyPressureShell: ',PressureOnShell

   return PressureOnShell    

  
  

def PrintMeshInfo(mesh,info):
   '''Calculate a variable pressure on shell elements'''


   from Utilitai.partition import *
   #import numpy

   PyMesh   = MAIL_PY()
   PyMesh.FromAster(mesh);
   nonu     = PyMesh.dime_maillage[0]
   elnu     = PyMesh.dime_maillage[2]
   NodeCoord    = PyMesh.cn
   ElemConnect  = PyMesh.co
   NodeList = list(PyMesh.correspondance_noeuds)
   ElemList = list(PyMesh.correspondance_mailles)
   ElemType = list(PyMesh.tm)
   NodeGroup = PyMesh.gno
   ElemGroup = PyMesh.gma

   print 'info set to %d'%(info)
   print 'print nodes and element data: '
   print 'node group: ',NodeGroup
   print 'number of nodes in group: ',nonu
   print 'number of elements in group: ',elnu
   shellcount = 0
   for ii in xrange(len(ElemList)):
     ##print 'elementtype: ',ii,ElementType[ii]
     if ElemType[ii]==16:  # select shell coque_3d elements 14:quad8, 16:quad9
       shellcount+=1
   print 'Number of shell elements: ',shellcount

   if info>=2:
     print 'info set to %d'%(info)+' - print additional information.'
     print PyMesh
     print 'node list: ',NodeList
     print 'element list: ',ElemList
     print 'element group: ',ElemGroup
     print 'element type list: ',ElemType

     print 'connection information for all elements: '
     for ii in xrange(len(ElemConnect)):
      tempstr = 'connection information of element %d:'%ii
      print tempstr,ElemConnect[ii]


   return



